"""Higher level module to handle data processing of BEA API results"""

import pandas as pd

from . import api

def json_result_to_dataframe(json_data) ->  pd.DataFrame:
    """This function converts a list (which we got from JSON) to a DataFrame,
    and takes care of the special case where list had 1 element, so a dict was returned
    """

    if isinstance(json_data, dict):
        df = pd.DataFrame(json_data, index=[0])
    else:
        df = pd.DataFrame(json_data)

    return df

class BEA(api.BEAAPI):
    r"""The primary class in the BEApy package, handles sending requests
    and manipulating the results. Needs to be initiated with your BEA
    API key once, it is stored for all later requests.

    Parameters
    ----------
    api_key
        Your BEA API key
    """

    def __init__(self, api_key: str) -> None:
        super().__init__(api_key)

    def dataset_list(self) -> pd.DataFrame:
        r"""Get a list of all availible datasets (API method ``getDatasetList``)

        Returns
        -------
        pd.DataFrame
            A DataFrame with a description and name of availible datasets.
        """

        params = {
            'method': 'GETDATASETLIST'
        }

        results = self.send_request(params)['Dataset']

        return json_result_to_dataframe(results)

    def parameter_list(self, dataset: str) -> pd.DataFrame:
        r"""Get a list of availible parameters for the dataset  (API method ``getParameterList``)

        Usually the parameter list will also include the value of the ALL option,
        as well as data type, whether the parameteris is required, and
        whether it accepts multiple values.

        Parameters
        ----------
        dataset
            The name of the dataset

        Returns
        -------
        pd.DataFrame
            A DataFrame with a description and name of availible parameters,
            as well as some other options.
        """

        params = {
            'method': 'GETPARAMETERLIST',
            'datasetName': dataset,
        }

        results = self.send_request(params)['Parameter']

        parameters = json_result_to_dataframe(results)

        # Shorten names
        parameters.columns = [x.replace('Parameter', '') for x in parameters.columns]
        parameters.columns = [x.replace('Flag', '') for x in parameters.columns]
        parameters['Name'] = parameters['Name'].str.strip()

        # Set name as index
        parameters.set_index('Name', inplace=True)

        # Default column order - filter columns not availible, append others
        default_order = ['Description', 'DefaultValue', 'DataType',
                         'IsRequired', 'MultipleAccepted', 'AllValue']
        order = [x for x in default_order if x in parameters.columns]
        extra = [x for x in parameters.columns if x not in order]

        order = order + extra

        # Reorder
        parameters = parameters[order]

        # Columns to bool
        parameters['MultipleAccepted'] = parameters['MultipleAccepted']\
            .astype(int).astype(bool)
        parameters['IsRequired'] = parameters['IsRequired']\
            .astype(int).astype(bool)

        return parameters

    def parameter_values(self, dataset: str, parameter: str, filtering: dict = None)-> pd.DataFrame:
        r"""Get a list of possible values for the parameter in a dataset (API method ``getParameterValues``).

        Optionally, filter the list based on values of other parameters (API method
        ``getParameterValuesFiltered``). Be careful in this case, as the filtered method is not
        implemented for all datasets.

        Parameters
        ----------
        dataset
            The name of the dataset
        parameter
            The name of the (target) parameter whose values you are retrieving
        filter
            A dictionary of parameter-value pairs, based on which to filter the target parameter.
            If set to None, `getParameterValues`` API method is used. Otherwise,
            ``getParameterValuesFiltered`` is used.

        Returns
        -------
        pd.DataFrame
            A DataFrame with a description and name (key) of availible parameters values
        """

        # Set up things differently, depending on whether we are filtering or not
        if filtering == None:
            method = 'GETPARAMETERVALUES'
            param_key = 'parameterName'
        else:
            # Throw error if filter is not a dictionary
            if type(filtering) != dict:
                raise TypeError('Filtering argument needs to be None or a dictionary')

            method = 'GETPARAMETERVALUESFILTERED'
            param_key = 'targetParameter'

        params = {
            'method': method,
            'datasetName': dataset,
            param_key: parameter
        }

        if filtering != None:
            params.update(filtering)

        results = self.send_request(params)['ParamValue']

        values = json_result_to_dataframe(results)

        return values
