import os

import pytest
import pandas as pd

from beapy import BEA
from beapy.api import BEAError


def test_wrong_id():
    """Test wrong ID, should catch a BEAError"""
    bea = BEA('WRONG-KEY')
    with pytest.raises(BEAError):
        bea.send_request({})

# From now on, the correct ID is used, so test can share a BEA object
@pytest.fixture(scope='session')
def api_key():
    return os.environ['BEA_API_KEY']


@pytest.fixture(scope='module')
def bea(api_key):
    bea = BEA(api_key)

    return bea


def test_wrong_param(bea):
    """Test wrong params, should catch a BEAError"""
    with pytest.raises(BEAError):
        bea.send_request({'WRONG PARAM': 0})


def test_wrong_method(bea):
    """Test wrong method name, should catch a BEAError"""
    with pytest.raises(BEAError):
        bea.send_request({'method': 'WRONG METHOD'})


@pytest.fixture(scope='module')
def dataset_list(bea):
    return bea.dataset_list()


class TestDatasetList(object):
    """This makes sure that the output of dataset_list() is what you
    would expect: a non-empty pd.DataFrame with 2 columns, one named
    'DatasetDescription', the other 'DatasetName'"""

    def test_type(self, dataset_list):
        assert isinstance(dataset_list, pd.DataFrame)

    def test_num_cols(self, dataset_list):
        assert dataset_list.shape[1] == 2

    def test_nonempty(self, dataset_list):
        assert dataset_list.shape[0] > 0

    def test_descr_column(self, dataset_list):
        assert 'DatasetDescription' in dataset_list.columns

    def test_name_column(self, dataset_list):
        assert 'DatasetName' in dataset_list.columns


class TestParameterList(object):
    """This makes sure that the output of parameter_list() if what we would
    expect for all the current databases (non-empty dataframe), and tests the
    presence of some parameters for the NIPA and MNE databases"""

    def test_wrong_database(self, bea):
        with pytest.raises(BEAError):
            bea.parameter_list('WRONG DATABASE')

    def test_default_datasets_general(self, bea, dataset_list):
        """This tests the output for all datasets, to make sure it is
        a DataFrame, not empty, and contains the Description columns"""

        db_list = list(dataset_list['DatasetName'])

        for dataset in db_list:
            params = bea.parameter_list(dataset)

            error_not_df = '{} parameter list is not a DataFrame'.format(dataset)
            error_empty = '{} parameter list is empty'.format(dataset)
            error_no_desc = '{} parameter list does not contain a Description column'.format(dataset)

            assert isinstance(params, pd.DataFrame), error_not_df
            assert params.shape[0] > 0, error_empty
            assert 'Description' in params.columns, error_no_desc

    def test_NIPA(self, bea):
        """This makes sure that NIPA contains the parameters and columns it should"""

        parameters = ['Frequency', 'ShowMillions', 'TableID', 'TableName', 'Year']
        columns = ['Description', 'DefaultValue', 'DataType', 'IsRequired', 'MultipleAccepted',
                   'AllValue']

        NIPA = bea.parameter_list('NIPA')

        assert set(parameters) == set(NIPA.index), 'Parameter list for NIPA not ok'
        assert set(columns) == set(NIPA.columns), 'Parameter attribute list (cols) for NIPA not ok'

    def test_MNE(self, bea):
        """This makes sure that MNE contains the parameters and columns it should"""

        parameters = ['DirectionOfInvestment', 'OwnershipLevel', 'NonbankAffiliatesOnly',
                      'Classification', 'Country', 'Industry', 'Year', 'State', 'SeriesID',
                      'GetFootnotes', 'Investment', 'ParentInvestment']
        columns = ['Description', 'DefaultValue', 'DataType', 'IsRequired', 'MultipleAccepted',
                   'AllValue']

        MNE = bea.parameter_list('MNE')

        assert set(parameters) == set(MNE.index), 'Parameter list for MNE not ok'
        assert set(columns) == set(MNE.columns), 'Parameter attribute list (cols) for MNE not ok'

class TestParameterValues(object):
    """This tests the parameter_values() method, testing that output is as expected for certain
    queries, and that filtering works well."""

    def test_wrong_database(self, bea):
        with pytest.raises(BEAError):
            bea.parameter_values(dataset='WRONG DATABASE', parameter='TABLENAME')

    def test_wrong_parameter(self, bea):
        with pytest.raises(BEAError):
            bea.parameter_values(dataset='NIPA', parameter='WRONG PARAMETER')

    def test_wrong_filtering(self, bea):
        """This tests that for a given database with filtering, we get an error
        if we do not use dictionary for filtering"""

        with pytest.raises(TypeError):
            bea.parameter_values(dataset='REGIONAL', parameter='LINECODE',
                                 filtering='WRONG')

    def test_wrong_filtering_database(self, bea):
        """This tests that for a given (actual) database without filtering, we get an error
        if we attempt to use filtering"""

        with pytest.raises(BEAError):
            bea.parameter_values(dataset='NIPA', parameter='YEAR',
                                 filtering={'FREQUENCY': 'Q'})

    #############################################################
    # Test normal return parameter values

    def test_ITA_indicator(self, bea):
        """Test that the values in ITA database for the Indicator parameter are ok"""

        true_vals = pd.read_csv('tests/Datasets/ITA_indicator.csv')
        get_vals = bea.parameter_values(dataset='ITA', parameter='INDICATOR')

        pd.testing.assert_frame_equal(true_vals, get_vals[true_vals.columns])

    def test_NIPA_table_name(self, bea):
        """Test that the values in NIPA database for the TableName parameter are ok"""

        true_vals = pd.read_csv('tests/Datasets/NIPA_table_name.csv')
        get_vals = bea.parameter_values(dataset='NIPA', parameter='TableName')

        pd.testing.assert_frame_equal(true_vals, get_vals[true_vals.columns])

    def test_FA_table_name(self, bea):
        """Test that the values in FixedAssets database for the TableName parameter are ok"""

        true_vals = pd.read_csv('tests/Datasets/FA_table_name.csv')
        get_vals = bea.parameter_values(dataset='FixedAssets', parameter='TableName')

        pd.testing.assert_frame_equal(true_vals, get_vals[true_vals.columns])

    def test_MNE_industry(self, bea):
        """Test that the values in MNE database for the Industry parameter are ok"""

        true_vals = pd.read_csv('tests/Datasets/MNE_industry.csv')
        get_vals = bea.parameter_values(dataset='MNE', parameter='Industry')

        pd.testing.assert_frame_equal(true_vals, get_vals[true_vals.columns])

    def test_IIP_TypeOfInvestment(self, bea):
        """Test that the values in IIP database for the TypeOfInvestment parameter are ok"""

        true_vals = pd.read_csv('tests/Datasets/IIP_TypeOfInvestment.csv')
        get_vals = bea.parameter_values(dataset='IIP', parameter='TypeOfInvestment')

        pd.testing.assert_frame_equal(true_vals, get_vals[true_vals.columns])

    def test_Regional_GeoFips(self, bea):
        """Test that the values in Regional database for the GeoFips parameter are ok"""

        true_vals = pd.read_csv('tests/Datasets/Regional_GeoFips.csv', dtype = {'Key': str})
        get_vals = bea.parameter_values(dataset='Regional', parameter='GeoFips')

        pd.testing.assert_frame_equal(true_vals, get_vals[true_vals.columns])

    #############################################################
    # Test filtering
    def test_Regional_LineCode_filter(self, bea):
        """Test that the values in Regional dataset for the LlineCode parameter filtered by
        TableName = SAINC1 are ok"""

        true_vals = pd.read_csv('tests/Datasets/Regional_LineCode_filtered.csv',
                                dtype = {'Key': str})
        get_vals = bea.parameter_values(dataset='Regional', parameter='LineCode',
                                        filtering = {'TABLENAME': 'SAINC1'})

        pd.testing.assert_frame_equal(true_vals, get_vals[true_vals.columns])

    def test_Regional_Year_filter(self, bea):
        """Test that the values in Regional dataset for the Year parameter filtered by
        TableName = CAINC5N and GEOFIPS = 01001 are ok"""

        true_vals = pd.read_csv('tests/Datasets/Regional_Year_filtered.csv',
                                dtype = {'Key': str, 'Desc': str})
        get_vals = bea.parameter_values(dataset='Regional', parameter='Year',
                                        filtering = {'TABLENAME': 'CAINC5N', 'GEOFIPS': '01001'})

        pd.testing.assert_frame_equal(true_vals, get_vals[true_vals.columns])
